searchData={"items":[{"type":"module","title":"Behave","doc":"~Ugh baby, `Behave`!\nSimple BDD style tests for elixir.","ref":"Behave.html"},{"type":"macro","title":"Behave.act/2","doc":"","ref":"Behave.html#act/2"},{"type":"macro","title":"Behave.check/2","doc":"","ref":"Behave.html#check/2"},{"type":"macro","title":"Behave.given/2","doc":"","ref":"Behave.html#given/2"},{"type":"macro","title":"Behave.scenario/2","doc":"","ref":"Behave.html#scenario/2"},{"type":"function","title":"Behave.string_to_function_name/1","doc":"","ref":"Behave.html#string_to_function_name/1"},{"type":"module","title":"Behave.Formatter","doc":"","ref":"Behave.Formatter.html"},{"type":"function","title":"Behave.Formatter.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Behave.Formatter.html#child_spec/1"},{"type":"module","title":"Behave.Formatter.HtmlRenderer","doc":"","ref":"Behave.Formatter.HtmlRenderer.html"},{"type":"function","title":"Behave.Formatter.HtmlRenderer.create_html_report/1","doc":"","ref":"Behave.Formatter.HtmlRenderer.html#create_html_report/1"},{"type":"function","title":"Behave.Formatter.HtmlRenderer.report_name/0","doc":"","ref":"Behave.Formatter.HtmlRenderer.html#report_name/0"},{"type":"module","title":"Behave.Formatter.Scenario","doc":"","ref":"Behave.Formatter.Scenario.html"},{"type":"module","title":"Behave.Formatter.Step","doc":"","ref":"Behave.Formatter.Step.html"},{"type":"module","title":"Behave.Formatter.Store","doc":"","ref":"Behave.Formatter.Store.html"},{"type":"function","title":"Behave.Formatter.Store.add_scenario/1","doc":"","ref":"Behave.Formatter.Store.html#add_scenario/1"},{"type":"function","title":"Behave.Formatter.Store.call/1","doc":"","ref":"Behave.Formatter.Store.html#call/1"},{"type":"function","title":"Behave.Formatter.Store.cast/2","doc":"","ref":"Behave.Formatter.Store.html#cast/2"},{"type":"function","title":"Behave.Formatter.Store.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Behave.Formatter.Store.html#child_spec/1"},{"type":"function","title":"Behave.Formatter.Store.create_report/0","doc":"","ref":"Behave.Formatter.Store.html#create_report/0"},{"type":"function","title":"Behave.Formatter.Store.start/0","doc":"","ref":"Behave.Formatter.Store.html#start/0"},{"type":"function","title":"Behave.Formatter.Store.update_status/1","doc":"","ref":"Behave.Formatter.Store.html#update_status/1"},{"type":"module","title":"Behave.Scenario","doc":"","ref":"Behave.Scenario.html"},{"type":"macro","title":"Behave.Scenario.act/2","doc":"","ref":"Behave.Scenario.html#act/2"},{"type":"macro","title":"Behave.Scenario.act/3","doc":"","ref":"Behave.Scenario.html#act/3"},{"type":"macro","title":"Behave.Scenario.act/4","doc":"","ref":"Behave.Scenario.html#act/4"},{"type":"macro","title":"Behave.Scenario.act/5","doc":"","ref":"Behave.Scenario.html#act/5"},{"type":"macro","title":"Behave.Scenario.check/2","doc":"","ref":"Behave.Scenario.html#check/2"},{"type":"macro","title":"Behave.Scenario.check/3","doc":"","ref":"Behave.Scenario.html#check/3"},{"type":"macro","title":"Behave.Scenario.check/4","doc":"","ref":"Behave.Scenario.html#check/4"},{"type":"macro","title":"Behave.Scenario.check/5","doc":"","ref":"Behave.Scenario.html#check/5"},{"type":"macro","title":"Behave.Scenario.given/2","doc":"","ref":"Behave.Scenario.html#given/2"},{"type":"macro","title":"Behave.Scenario.given/3","doc":"","ref":"Behave.Scenario.html#given/3"},{"type":"macro","title":"Behave.Scenario.given/4","doc":"","ref":"Behave.Scenario.html#given/4"},{"type":"function","title":"Behave.Scenario.new/0","doc":"","ref":"Behave.Scenario.html#new/0"},{"type":"function","title":"Behave.Scenario.run/1","doc":"","ref":"Behave.Scenario.html#run/1"},{"type":"function","title":"Behave.Scenario.run_step/2","doc":"","ref":"Behave.Scenario.html#run_step/2"},{"type":"type","title":"Behave.Scenario.t/0","doc":"","ref":"Behave.Scenario.html#t:t/0"},{"type":"extras","title":"Behave","doc":"# Behave\n> Behaviour driven development for ExUnit with product management friendly readable scenarios.\n\n![Behave, baby, behave!](https://i.giphy.com/3o7bu1iM5MSwG2y7NS.gif)","ref":"readme.html"},{"type":"extras","title":"Features - Behave","doc":"* Minimal DSL on top of ExUnit.\n* given, act, check instead of given, when, then to avoid naming collisions.\n* Api can be used without macros, if necessary\n* Steps are just functions, scenarios are just ExUnit tests.\n* Not a full gherkin implementation, it just gives you the ability to decompose tests into reusable named steps.\n* Separate DSLs for defining scenarios and implementing test steps.\n* Dependency Free. Use whatever assertion, mocking and testing libraries you want.\n* Html report including all test steps.","ref":"readme.html#features"},{"type":"extras","title":"Usage - Behave","doc":"Assuming you have a BDD style scenario like this:\n\n```Gherkin\n\nScenario make coffee\n  Given a coffee machine\n  And it has 250 ml of water in its tank\n  And it has java coffee beans in its reservoir\n  When i press the \"make coffee\" button\n  Then it makes coffee\n\n```\n\nIn your ExUnit test, use the `Behave` module:\n\n```elixir\n\ndefmodule MyCoffeeMachineTest do\n  use ExUnit.Case\n  use Behave, steps: [MyCoffeeMachineTestSteps]\n\n  ...\nend\n\n```\n\nNotice we're passing `:steps` to `Behave`, this is the module where you define your test steps.\n\nThen, use the `Behave` DSL to implement your scenario:\n\n```elixir\n\ndefmodule MyCoffeeMachineTest do\n  use ExUnit.Case\n  use Behave, steps: [MyCoffeeMachineTestSteps]\n\n  scenario \"make coffee with dsl\" do\n    given \"coffee machine\"\n    given \"it has water\", amount: 250\n    given \"it has coffee\", cultivar: :java\n    act \"i press the button\"\n    check \"it makes coffee\"\n  end\nend\n\n```\nYour `TestSteps` modules should be defined in a directory that gets picked up by the compiler, such as `test/support`.\nIn those modules, you need to define a step function for each test step (given, act, check).\n\n```elixir\n\ndefmodule TestSteps do\n    use Behave.Scenario # for access to the step definition DSL\n    import ExUnit.Assertions\n\n    given \"coffee machine\" do\n      {:coffee_machine, CoffeeMachine.new()}\n    end\n\n    ...\nend\n\n```\n\nThe `given` macro expects you to return a tuple of an atom that will be used to refer to the returned value, and the value that should be available to subsequent steps.\nYou can run arbitrary code before returning the tuple. If you do not need the `given` steps to produce any values, any return value that is not wrapped in a tuple will be ignored.\nIn a `given` step, you have access to the values that other `givens` have emitted via the second argument. All macros come with several variants with different arities and guard clauses to access values emitted from previous steps and arguments passed in from the scenario. The arguments are always a keyword list and always the last argument. \n\n```elixir\n# in scenario:\n\ngiven \"it has water\", amount: 250\n\n# in step definition:\n\ngiven \"it has water\", data, amount: amount do\n  {:coffee_machine, CoffeeMachine.add_water(data.coffee_machine, amount)}\nend\n\n```\n\nBecause extracting a value, processing it and then reassigning it to the same kay is a common pattern, there is a shorthand notation for it.\nIf you pass in a function as the second argument of the tuple, the function will be executed, and the value of the key will be passed in, if it has been set by a prior step.\nThe value will be overwritten with this function's return value, similar to how `update_in` works.\n\n```elixir\n\ngiven \"it has water\", amount: amount do\n  {:coffee_machine, &CoffeeMachine.add_water(&1, amount)}\nend\n\n```\n\n`act` works the same. The main difference is that tuples returned from `act` steps will be assigned to a different map than those from `givens`. This is to separate the prerequisites of a test from its results, and to make comparisons between what was and what is easier.\n`act` can accept arguments, just as `given` can.\n\n```elixir\n\nact \"i press the button\", data do\n  {:coffee, CoffeeMachine.brew(data.coffee_machine)}\nend\n\n```\nNote that it is not possible to overwrite values set in `givens`. If you need to modify a value created by a `given`, use a `given`. \n\nThe passed in data is a `Map`. You can also use pattern matching to extract values in a concise way:\n\n```elixir\n\ngiven \"it has water\", %{coffee_machine: it}, amount: amount do\n  {:coffee_machine, CoffeeMachine.add_water(it, amount)}\nend\n\nact \"i press the button\", %{coffee_machine: it} do\n  {:coffee, CoffeeMachine.brew(it)}\nend\n\n```\n\nFinally, run your assertions in a `check` step:\n\n```elixir\n\ncheck \"it makes coffee\", results do\n  assert results.coffee != :disappointment\nend\n\n```\n\n`check`s cannot store any values. Their return values are always discarded. However, they have access to both the `data` and `results` maps. \nThe values produced in the `act` steps are available in a `check` step through its first argument. \nIf you need to access the values emitted in `given`, they are passed in as the second argument.\n`data` contains all the values set in `given` steps, and `results` contains the values from the `act` steps.\n\n> Note that `given` and `act` receives `data` as its first argument, but `check` receives `results` as its first argument. This is done to make the common case less verbose to write. \n\n```elixir\n\ncheck \"it makes coffee, when given water and coffee\", results, data do\n  assert results.coffee != :disappointment\n  assert data.coffee_machine.water_ml != 0\n  assert data.coffee_machine.coffee != nil\nend\n\n```\nIf you need to access `data`, but not `results`, just assign `data` to `_`:\n\n```elixir\n\ncheck \"there's still water and coffee in the machine after brewing\", _, data do\n  assert data.coffee_machine.water_ml != 0\n  assert data.coffee_machine.coffee != nil\nend\n\n```\n\n`check` can also accept arguments passed in from the scenario. These are expected to be keyword lists and will always be available as the last argument.\n\n```elixir\n\ncheck \"it makes the right kind of coffee\", results, coffee_variant: variant do\n  {:coffee, cup} = results.coffee\n  assert cup |> String.contains(variant)\nend\n\ncheck \"after making coffee, the tank isnt empty\", results, data, tolerable_water_remaining: rest_ml do\n  assert results.coffee != :disappointment\n  assert data.coffee_machine.water_ml >= rest_ml\nend\n\n```\n\nThe tests created by the macros are just ExUnit tests.\nTo run your tests, just `mix test` them 🎉","ref":"readme.html#usage"},{"type":"extras","title":"HTML Report - Behave","doc":"Behave has a reporter for `ExUnit` that generates a html report from your tests. \nTo use it, add it to your reporters like so\n\n```elixir\n# test/test_helper.exs\n\nBehave.Formatter.Store.start() # add\nExUnit.configure(formatters: [ExUnit.CLIFormatter, Behave.Formatter]) # add\n\nExUnit.start()\n\n```\nand add a configuration value that tells it what to name the report file:\n\n```elixir\n# config/config.exs\nconfig :behave,\n    report_name: \"my_report_name.html\"\n\n```","ref":"readme.html#html-report"},{"type":"extras","title":"Migrating from 0.1.x - Behave","doc":"* It is no longer necessary to return `:ignore` from `given` and `act`. Any value returned that is not wrapped in a tuple will be discarded.\n* The implicitly-available `data` and `results` variables no longer exist. You now have to explicitly state that you want to access them in the arguments to a step. \n* If you want to access a value from an earlier `given` in a `given` step, you no longer _have_ to use a lambda. You still can, as a shorthand, but it's no longer the only or preferred way. This should reduce the need to wrap the values emitted in `given` steps in further maps as well as \"hacky\" solutions making use of lambdas to access one key and emit into another.\n\n#","ref":"readme.html#migrating-from-0-1-x"},{"type":"extras","title":"How to refactor - Behave","doc":"1) find all usages of `data` and `results`, and explicitly add them to the step arguments. \n2) Any arguments you already have there, passing in values from the scenarios, should be in the form of a keyword list and always in the last position. Passing in anything but a keyword list will not work, but the macros are smart enough to figure out if you want to access any combination of `data`, `results` and scenario arguments.\n3) Optionally, get rid of any hacks to work around not having access to `data` in `givens`. Those are still supported, but no longer necessary.","ref":"readme.html#how-to-refactor"},{"type":"extras","title":"Installation - Behave","doc":"The package is available on [hex.pm](https://hex.pm/packages/behave_bdd) and can be installed\nby adding `behave_bdd` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:behave, \"~> 0.2.0\", hex: :behave_bdd}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#installation"}],"content_type":"text/markdown"}